from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response
from ... import errors

from typing import Dict
from ...models.study import Study


def _get_kwargs(
    id: str,
    *,
    authorization: str,
) -> Dict[str, Any]:
    headers = {}
    headers["Authorization"] = authorization

    return {
        "method": "get",
        "url": "/api/v1/studies/{id}/".format(
            id=id,
        ),
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Study]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Study.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Study]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    authorization: str,
) -> Response[Study]:
    """Retrieve a study

     Retrieve a study by id. If you are polling the API for updates to a study, consider using a
    [Hook](#tag/Hooks). We will call your endpoint when certain events occur on your study, such as new
    completed submissions or changes in status.

    Args:
        id (str):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Study]
    """

    kwargs = _get_kwargs(
        id=id,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: str,
    *,
    client: AuthenticatedClient,
    authorization: str,
) -> Optional[Study]:
    """Retrieve a study

     Retrieve a study by id. If you are polling the API for updates to a study, consider using a
    [Hook](#tag/Hooks). We will call your endpoint when certain events occur on your study, such as new
    completed submissions or changes in status.

    Args:
        id (str):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Study
    """

    return sync_detailed(
        id=id,
        client=client,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    authorization: str,
) -> Response[Study]:
    """Retrieve a study

     Retrieve a study by id. If you are polling the API for updates to a study, consider using a
    [Hook](#tag/Hooks). We will call your endpoint when certain events occur on your study, such as new
    completed submissions or changes in status.

    Args:
        id (str):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Study]
    """

    kwargs = _get_kwargs(
        id=id,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: str,
    *,
    client: AuthenticatedClient,
    authorization: str,
) -> Optional[Study]:
    """Retrieve a study

     Retrieve a study by id. If you are polling the API for updates to a study, consider using a
    [Hook](#tag/Hooks). We will call your endpoint when certain events occur on your study, such as new
    completed submissions or changes in status.

    Args:
        id (str):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Study
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            authorization=authorization,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING


from attrs import define as _attrs_define


from typing import List
from typing import Dict

if TYPE_CHECKING:
    from ..models.mutually_exclusive_study_collection_update import (
        MutuallyExclusiveStudyCollectionUpdate,
    )


T = TypeVar("T", bound="MutuallyExclusiveStudyCollectionsResponse")


@_attrs_define
class MutuallyExclusiveStudyCollectionsResponse:
    """
    Attributes:
        results (List['MutuallyExclusiveStudyCollectionUpdate']): List of all mutually exclusive study collections in a
            project
    """

    results: List["MutuallyExclusiveStudyCollectionUpdate"]

    def to_dict(self) -> Dict[str, Any]:
        results = []
        for results_item_data in self.results:
            results_item = results_item_data.to_dict()

            results.append(results_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "results": results,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.mutually_exclusive_study_collection_update import (
            MutuallyExclusiveStudyCollectionUpdate,
        )

        d = src_dict.copy()
        results = []
        _results = d.pop("results")
        for results_item_data in _results:
            results_item = MutuallyExclusiveStudyCollectionUpdate.from_dict(
                results_item_data
            )

            results.append(results_item)

        mutually_exclusive_study_collections_response = cls(
            results=results,
        )

        return mutually_exclusive_study_collections_response



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response
from ... import errors

from ...models.request_submission_return_json_body import (
    RequestSubmissionReturnJsonBody,
)
from typing import Dict
from ...models.return_requested_response import ReturnRequestedResponse


def _get_kwargs(
    id: str,
    *,
    json_body: RequestSubmissionReturnJsonBody,
) -> Dict[str, Any]:
    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": "/api/v1/submissions/{id}/request-return/".format(
            id=id,
        ),
        "json": json_json_body,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[ReturnRequestedResponse]:
    if response.status_code == HTTPStatus.OK:
        response_200 = ReturnRequestedResponse.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[ReturnRequestedResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: RequestSubmissionReturnJsonBody,
) -> Response[ReturnRequestedResponse]:
    r"""Request the participant who submitted the response to return their response

     **This is an experimental feature that may be subject to change in the future.** <br/>
    It offers researchers the ability to ask a participant to return a submission. The return reason
    must be provided in the request and can be any free text string. <br/> The Prolific UI allows users
    to select any of the following options:
    * Didn't finish the study
    * Encountered technical problems
    * Withdrew consent
    * Other ( uses the free text input)

    This constructs a message around the reasons provided so there is no need to provide additional text
    beyond the reasons.
    <img alt=\"Example\" src=\"/assets/img/api/message.png\" />

    Args:
        id (str):
        json_body (RequestSubmissionReturnJsonBody):  Example: {'request_return_reasons':
            ['Withdrew consent.', 'Did not finish study.']}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ReturnRequestedResponse]
    """

    kwargs = _get_kwargs(
        id=id,
        json_body=json_body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: RequestSubmissionReturnJsonBody,
) -> Optional[ReturnRequestedResponse]:
    r"""Request the participant who submitted the response to return their response

     **This is an experimental feature that may be subject to change in the future.** <br/>
    It offers researchers the ability to ask a participant to return a submission. The return reason
    must be provided in the request and can be any free text string. <br/> The Prolific UI allows users
    to select any of the following options:
    * Didn't finish the study
    * Encountered technical problems
    * Withdrew consent
    * Other ( uses the free text input)

    This constructs a message around the reasons provided so there is no need to provide additional text
    beyond the reasons.
    <img alt=\"Example\" src=\"/assets/img/api/message.png\" />

    Args:
        id (str):
        json_body (RequestSubmissionReturnJsonBody):  Example: {'request_return_reasons':
            ['Withdrew consent.', 'Did not finish study.']}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ReturnRequestedResponse
    """

    return sync_detailed(
        id=id,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: RequestSubmissionReturnJsonBody,
) -> Response[ReturnRequestedResponse]:
    r"""Request the participant who submitted the response to return their response

     **This is an experimental feature that may be subject to change in the future.** <br/>
    It offers researchers the ability to ask a participant to return a submission. The return reason
    must be provided in the request and can be any free text string. <br/> The Prolific UI allows users
    to select any of the following options:
    * Didn't finish the study
    * Encountered technical problems
    * Withdrew consent
    * Other ( uses the free text input)

    This constructs a message around the reasons provided so there is no need to provide additional text
    beyond the reasons.
    <img alt=\"Example\" src=\"/assets/img/api/message.png\" />

    Args:
        id (str):
        json_body (RequestSubmissionReturnJsonBody):  Example: {'request_return_reasons':
            ['Withdrew consent.', 'Did not finish study.']}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ReturnRequestedResponse]
    """

    kwargs = _get_kwargs(
        id=id,
        json_body=json_body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: RequestSubmissionReturnJsonBody,
) -> Optional[ReturnRequestedResponse]:
    r"""Request the participant who submitted the response to return their response

     **This is an experimental feature that may be subject to change in the future.** <br/>
    It offers researchers the ability to ask a participant to return a submission. The return reason
    must be provided in the request and can be any free text string. <br/> The Prolific UI allows users
    to select any of the following options:
    * Didn't finish the study
    * Encountered technical problems
    * Withdrew consent
    * Other ( uses the free text input)

    This constructs a message around the reasons provided so there is no need to provide additional text
    beyond the reasons.
    <img alt=\"Example\" src=\"/assets/img/api/message.png\" />

    Args:
        id (str):
        json_body (RequestSubmissionReturnJsonBody):  Example: {'request_return_reasons':
            ['Withdrew consent.', 'Did not finish study.']}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ReturnRequestedResponse
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            json_body=json_body,
        )
    ).parsed



from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response
from ... import errors

from typing import cast
from ...models.participant_i_ds import ParticipantIDs
from typing import cast, Union
from ...models.submission_i_ds import SubmissionIDs
from typing import Dict


def _get_kwargs(
    *,
    json_body: Union["ParticipantIDs", "SubmissionIDs"],
    authorization: str,
) -> Dict[str, Any]:
    headers = {}
    headers["Authorization"] = authorization

    json_json_body: Dict[str, Any]

    if isinstance(json_body, ParticipantIDs):
        json_json_body = json_body.to_dict()

    else:
        json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": "/api/v1/submissions/bulk-approve/",
        "json": json_json_body,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[str]:
    if response.status_code == HTTPStatus.OK:
        response_200 = cast(str, response.json())
        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[str]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    json_body: Union["ParticipantIDs", "SubmissionIDs"],
    authorization: str,
) -> Response[str]:
    """Bulk approve submissions

     Bulk approve study submissions to pay participants after they have
    completed your survey or experiment. There are two variant payloads to
    this request.

    1. You can supply a Study ID, and a list of participant IDs, or
    2. You can provide a list of submission IDs

    Args:
        authorization (str):
        json_body (Union['ParticipantIDs', 'SubmissionIDs']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[str]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    json_body: Union["ParticipantIDs", "SubmissionIDs"],
    authorization: str,
) -> Optional[str]:
    """Bulk approve submissions

     Bulk approve study submissions to pay participants after they have
    completed your survey or experiment. There are two variant payloads to
    this request.

    1. You can supply a Study ID, and a list of participant IDs, or
    2. You can provide a list of submission IDs

    Args:
        authorization (str):
        json_body (Union['ParticipantIDs', 'SubmissionIDs']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        str
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    json_body: Union["ParticipantIDs", "SubmissionIDs"],
    authorization: str,
) -> Response[str]:
    """Bulk approve submissions

     Bulk approve study submissions to pay participants after they have
    completed your survey or experiment. There are two variant payloads to
    this request.

    1. You can supply a Study ID, and a list of participant IDs, or
    2. You can provide a list of submission IDs

    Args:
        authorization (str):
        json_body (Union['ParticipantIDs', 'SubmissionIDs']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[str]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    json_body: Union["ParticipantIDs", "SubmissionIDs"],
    authorization: str,
) -> Optional[str]:
    """Bulk approve submissions

     Bulk approve study submissions to pay participants after they have
    completed your survey or experiment. There are two variant payloads to
    this request.

    1. You can supply a Study ID, and a list of participant IDs, or
    2. You can provide a list of submission IDs

    Args:
        authorization (str):
        json_body (Union['ParticipantIDs', 'SubmissionIDs']):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        str
    """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
            authorization=authorization,
        )
    ).parsed



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.filter_set_list import FilterSetList
from ...types import UNSET, Unset
from typing import Union
from typing import Optional
from typing import Dict


def _get_kwargs(
    *,
    workspace_id: Union[Unset, None, str] = UNSET,
    organisation_id: Union[Unset, None, str] = UNSET,
    authorization: str,
) -> Dict[str, Any]:
    headers = {}
    headers["Authorization"] = authorization

    params: Dict[str, Any] = {}
    params["workspace_id"] = workspace_id

    params["organisation_id"] = organisation_id

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/api/v1/filter-sets/",
        "params": params,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[FilterSetList]:
    if response.status_code == HTTPStatus.OK:
        response_200 = FilterSetList.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[FilterSetList]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    workspace_id: Union[Unset, None, str] = UNSET,
    organisation_id: Union[Unset, None, str] = UNSET,
    authorization: str,
) -> Response[FilterSetList]:
    """List all filter sets

     List of all filter sets in the specified workspace.

    Args:
        workspace_id (Union[Unset, None, str]):
        organisation_id (Union[Unset, None, str]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[FilterSetList]
    """

    kwargs = _get_kwargs(
        workspace_id=workspace_id,
        organisation_id=organisation_id,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    workspace_id: Union[Unset, None, str] = UNSET,
    organisation_id: Union[Unset, None, str] = UNSET,
    authorization: str,
) -> Optional[FilterSetList]:
    """List all filter sets

     List of all filter sets in the specified workspace.

    Args:
        workspace_id (Union[Unset, None, str]):
        organisation_id (Union[Unset, None, str]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        FilterSetList
    """

    return sync_detailed(
        client=client,
        workspace_id=workspace_id,
        organisation_id=organisation_id,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    workspace_id: Union[Unset, None, str] = UNSET,
    organisation_id: Union[Unset, None, str] = UNSET,
    authorization: str,
) -> Response[FilterSetList]:
    """List all filter sets

     List of all filter sets in the specified workspace.

    Args:
        workspace_id (Union[Unset, None, str]):
        organisation_id (Union[Unset, None, str]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[FilterSetList]
    """

    kwargs = _get_kwargs(
        workspace_id=workspace_id,
        organisation_id=organisation_id,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    workspace_id: Union[Unset, None, str] = UNSET,
    organisation_id: Union[Unset, None, str] = UNSET,
    authorization: str,
) -> Optional[FilterSetList]:
    """List all filter sets

     List of all filter sets in the specified workspace.

    Args:
        workspace_id (Union[Unset, None, str]):
        organisation_id (Union[Unset, None, str]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        FilterSetList
    """

    return (
        await asyncio_detailed(
            client=client,
            workspace_id=workspace_id,
            organisation_id=organisation_id,
            authorization=authorization,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
import datetime
from ..models.return_requested_response_status import ReturnRequestedResponseStatus
from ..types import UNSET, Unset
from dateutil.parser import isoparse


T = TypeVar("T", bound="ReturnRequestedResponse")


@_attrs_define
class ReturnRequestedResponse:
    """
    Attributes:
        id (Union[Unset, str]): the database id of the submission instance
        status (Union[Unset, ReturnRequestedResponseStatus]): The current status of the submission
        participant (Union[Unset, str]): The participant who took part in the study.
        return_requested (Union[Unset, None, datetime.datetime]): The date and time when a request was made to return a
            submission.
    """

    id: Union[Unset, str] = UNSET
    status: Union[Unset, ReturnRequestedResponseStatus] = UNSET
    participant: Union[Unset, str] = UNSET
    return_requested: Union[Unset, None, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        participant = self.participant
        return_requested: Union[Unset, None, str] = UNSET
        if not isinstance(self.return_requested, Unset):
            return_requested = (
                self.return_requested.isoformat() if self.return_requested else None
            )

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if status is not UNSET:
            field_dict["status"] = status
        if participant is not UNSET:
            field_dict["participant"] = participant
        if return_requested is not UNSET:
            field_dict["return_requested"] = return_requested

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ReturnRequestedResponseStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = ReturnRequestedResponseStatus(_status)

        participant = d.pop("participant", UNSET)

        _return_requested = d.pop("return_requested", UNSET)
        return_requested: Union[Unset, None, datetime.datetime]
        if _return_requested is None:
            return_requested = None
        elif isinstance(_return_requested, Unset):
            return_requested = UNSET
        else:
            return_requested = isoparse(_return_requested)

        return_requested_response = cls(
            id=id,
            status=status,
            participant=participant,
            return_requested=return_requested,
        )

        return_requested_response.additional_properties = d
        return return_requested_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TransitionMutuallyExclusiveStudyCollectionJsonBodyAction(str, Enum):
    CANCEL_PUBLISH = "CANCEL_PUBLISH"
    PUBLISH = "PUBLISH"
    SCHEDULE_PUBLISH = "SCHEDULE_PUBLISH"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response
from ... import errors

from typing import Dict
from ...models.base_study import BaseStudy
from ...models.study import Study


def _get_kwargs(
    id: str,
    *,
    json_body: BaseStudy,
    authorization: str,
) -> Dict[str, Any]:
    headers = {}
    headers["Authorization"] = authorization

    json_json_body = json_body.to_dict()

    return {
        "method": "patch",
        "url": "/api/v1/studies/{id}/".format(
            id=id,
        ),
        "json": json_json_body,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Study]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Study.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Study]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: BaseStudy,
    authorization: str,
) -> Response[Study]:
    """Update a study

     You can update any field for a draft study.

    Once the study has been published only the following fields can be updated with some restrictions:
    - internal_name: Internal name of the study, not shown to participants
    - total_available_places: Only increasing is allowed. A completed study will become active again and
    resume recruiting of participants. For more information, check the [guide](https://researcher-
    help.prolific.com/hc/en-gb/articles/360009222553)

    Args:
        id (str):
        authorization (str):
        json_body (BaseStudy):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Study]
    """

    kwargs = _get_kwargs(
        id=id,
        json_body=json_body,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: BaseStudy,
    authorization: str,
) -> Optional[Study]:
    """Update a study

     You can update any field for a draft study.

    Once the study has been published only the following fields can be updated with some restrictions:
    - internal_name: Internal name of the study, not shown to participants
    - total_available_places: Only increasing is allowed. A completed study will become active again and
    resume recruiting of participants. For more information, check the [guide](https://researcher-
    help.prolific.com/hc/en-gb/articles/360009222553)

    Args:
        id (str):
        authorization (str):
        json_body (BaseStudy):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Study
    """

    return sync_detailed(
        id=id,
        client=client,
        json_body=json_body,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: BaseStudy,
    authorization: str,
) -> Response[Study]:
    """Update a study

     You can update any field for a draft study.

    Once the study has been published only the following fields can be updated with some restrictions:
    - internal_name: Internal name of the study, not shown to participants
    - total_available_places: Only increasing is allowed. A completed study will become active again and
    resume recruiting of participants. For more information, check the [guide](https://researcher-
    help.prolific.com/hc/en-gb/articles/360009222553)

    Args:
        id (str):
        authorization (str):
        json_body (BaseStudy):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Study]
    """

    kwargs = _get_kwargs(
        id=id,
        json_body=json_body,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: BaseStudy,
    authorization: str,
) -> Optional[Study]:
    """Update a study

     You can update any field for a draft study.

    Once the study has been published only the following fields can be updated with some restrictions:
    - internal_name: Internal name of the study, not shown to participants
    - total_available_places: Only increasing is allowed. A completed study will become active again and
    resume recruiting of participants. For more information, check the [guide](https://researcher-
    help.prolific.com/hc/en-gb/articles/360009222553)

    Args:
        id (str):
        authorization (str):
        json_body (BaseStudy):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Study
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            json_body=json_body,
            authorization=authorization,
        )
    ).parsed



from enum import Enum


class GetStudiesState(str, Enum):
    ACTIVE = "ACTIVE"
    AWAITING_REVIEW = "AWAITING REVIEW"
    COMPLETED = "COMPLETED"
    PAUSED = "PAUSED"
    PUBLISHING = "PUBLISHING"
    SCHEDULED = "SCHEDULED"
    UNKNOWN = "UNKNOWN"
    UNPUBLISHED = "UNPUBLISHED"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="FilterSetParticipantCount")


@_attrs_define
class FilterSetParticipantCount:
    """
    Attributes:
        eligible_participant_count (Union[Unset, int]): The number of participants who match the filter sets filters.
            Please note that if the number is
            lower than 25 the count will be obscured to prevent identification of participants.
    """

    eligible_participant_count: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        eligible_participant_count = self.eligible_participant_count

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if eligible_participant_count is not UNSET:
            field_dict["eligible_participant_count"] = eligible_participant_count

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        eligible_participant_count = d.pop("eligible_participant_count", UNSET)

        filter_set_participant_count = cls(
            eligible_participant_count=eligible_participant_count,
        )

        filter_set_participant_count.additional_properties = d
        return filter_set_participant_count

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field


from typing import cast, List


T = TypeVar("T", bound="RequestSubmissionReturnJsonBody")


@_attrs_define
class RequestSubmissionReturnJsonBody:
    """
    Example:
        {'request_return_reasons': ['Withdrew consent.', 'Did not finish study.']}

    Attributes:
        request_return_reasons (List[str]):
    """

    request_return_reasons: List[str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        request_return_reasons = self.request_return_reasons

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "request_return_reasons": request_return_reasons,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        request_return_reasons = cast(List[str], d.pop("request_return_reasons"))

        request_submission_return_json_body = cls(
            request_return_reasons=request_return_reasons,
        )

        request_submission_return_json_body.additional_properties = d
        return request_submission_return_json_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...types import UNSET, Unset
from typing import Optional
from typing import Union


def _get_kwargs(
    *,
    researcher_id: str,
    offset: Union[Unset, None, int] = 0,
    limit: Union[Unset, None, int] = 100,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}
    params["researcher_id"] = researcher_id

    params["offset"] = offset

    params["limit"] = limit

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/api/v1/surveys/",
        "params": params,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Any]:
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    researcher_id: str,
    offset: Union[Unset, None, int] = 0,
    limit: Union[Unset, None, int] = 100,
) -> Response[Any]:
    """Get all surveys

     Get all the surveys for a researcher.

    Args:
        researcher_id (str):
        offset (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):  Default: 100.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        researcher_id=researcher_id,
        offset=offset,
        limit=limit,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    researcher_id: str,
    offset: Union[Unset, None, int] = 0,
    limit: Union[Unset, None, int] = 100,
) -> Response[Any]:
    """Get all surveys

     Get all the surveys for a researcher.

    Args:
        researcher_id (str):
        offset (Union[Unset, None, int]):
        limit (Union[Unset, None, int]):  Default: 100.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        researcher_id=researcher_id,
        offset=offset,
        limit=limit,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response
from ... import errors

from ...models.workspace import Workspace
from typing import Dict


def _get_kwargs(
    workspace_id: str,
    *,
    json_body: Workspace,
    authorization: str,
) -> Dict[str, Any]:
    headers = {}
    headers["Authorization"] = authorization

    json_json_body = json_body.to_dict()

    return {
        "method": "patch",
        "url": "/api/v1/workspaces/{workspace_id}/".format(
            workspace_id=workspace_id,
        ),
        "json": json_json_body,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Workspace]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Workspace.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Workspace]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    workspace_id: str,
    *,
    client: AuthenticatedClient,
    json_body: Workspace,
    authorization: str,
) -> Response[Workspace]:
    """Update a workspace

     Updates a workspace's details.

    Args:
        workspace_id (str):
        authorization (str):
        json_body (Workspace):  Example: {'id': '62fce6fff0a78eb4f3ebc09c', 'title': 'My
            workspace', 'description': 'This workspace does...', 'owner': '60a42f4c693c29420793cb73',
            'users': [{'id': '60a42f4c693c29420793cb73', 'name': 'Joe Soap', 'email':
            'joe.soap@gmail.com', 'roles': ['WORKSPACE_ADMIN']}], 'projects': [{'id':
            '60a42f4c693c29420793cb73'}], 'wallet': '61a65c06b084910b3f0c00d6'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Workspace]
    """

    kwargs = _get_kwargs(
        workspace_id=workspace_id,
        json_body=json_body,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    workspace_id: str,
    *,
    client: AuthenticatedClient,
    json_body: Workspace,
    authorization: str,
) -> Optional[Workspace]:
    """Update a workspace

     Updates a workspace's details.

    Args:
        workspace_id (str):
        authorization (str):
        json_body (Workspace):  Example: {'id': '62fce6fff0a78eb4f3ebc09c', 'title': 'My
            workspace', 'description': 'This workspace does...', 'owner': '60a42f4c693c29420793cb73',
            'users': [{'id': '60a42f4c693c29420793cb73', 'name': 'Joe Soap', 'email':
            'joe.soap@gmail.com', 'roles': ['WORKSPACE_ADMIN']}], 'projects': [{'id':
            '60a42f4c693c29420793cb73'}], 'wallet': '61a65c06b084910b3f0c00d6'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Workspace
    """

    return sync_detailed(
        workspace_id=workspace_id,
        client=client,
        json_body=json_body,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    workspace_id: str,
    *,
    client: AuthenticatedClient,
    json_body: Workspace,
    authorization: str,
) -> Response[Workspace]:
    """Update a workspace

     Updates a workspace's details.

    Args:
        workspace_id (str):
        authorization (str):
        json_body (Workspace):  Example: {'id': '62fce6fff0a78eb4f3ebc09c', 'title': 'My
            workspace', 'description': 'This workspace does...', 'owner': '60a42f4c693c29420793cb73',
            'users': [{'id': '60a42f4c693c29420793cb73', 'name': 'Joe Soap', 'email':
            'joe.soap@gmail.com', 'roles': ['WORKSPACE_ADMIN']}], 'projects': [{'id':
            '60a42f4c693c29420793cb73'}], 'wallet': '61a65c06b084910b3f0c00d6'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Workspace]
    """

    kwargs = _get_kwargs(
        workspace_id=workspace_id,
        json_body=json_body,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    workspace_id: str,
    *,
    client: AuthenticatedClient,
    json_body: Workspace,
    authorization: str,
) -> Optional[Workspace]:
    """Update a workspace

     Updates a workspace's details.

    Args:
        workspace_id (str):
        authorization (str):
        json_body (Workspace):  Example: {'id': '62fce6fff0a78eb4f3ebc09c', 'title': 'My
            workspace', 'description': 'This workspace does...', 'owner': '60a42f4c693c29420793cb73',
            'users': [{'id': '60a42f4c693c29420793cb73', 'name': 'Joe Soap', 'email':
            'joe.soap@gmail.com', 'roles': ['WORKSPACE_ADMIN']}], 'projects': [{'id':
            '60a42f4c693c29420793cb73'}], 'wallet': '61a65c06b084910b3f0c00d6'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Workspace
    """

    return (
        await asyncio_detailed(
            workspace_id=workspace_id,
            client=client,
            json_body=json_body,
            authorization=authorization,
        )
    ).parsed



from enum import Enum


class StudyShortStudyType(str, Enum):
    SINGLE = "SINGLE"
    UK_REP_SAMPLE = "UK_REP_SAMPLE"
    US_REP_SAMPLE = "US_REP_SAMPLE"
    QUOTA = "QUOTA"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class BaseStudyDeviceCompatibilityItem(str, Enum):
    DESKTOP = "desktop"
    MOBILE = "mobile"
    TABLET = "tablet"

    def __str__(self) -> str:
        return str(self.value)
